import pystan
import matplotlib.pyplot as plt
import numpy as np

OneVar_code = """
data {
        int<lower=0> J;
        real y[J];
}
parameters {
        real<lower=-1000, upper=1000> mu;
        real<lower=0, upper=1000> sigma;
}
transformed parameters {
}
model {
        y ~ normal(mu, sigma);
}
"""

LData =  [ 44, 44, 56, 59, 44, 44, 63, 60, 34, 58,
           55, 50, 32, 61, 47, 25, 38, 47, 30, 42]

OneVar_dat = { 'J' : len(LData), 'y': LData }

sm = pystan.StanModel(model_code=OneVar_code)
fit = sm.sampling(data=OneVar_dat, iter=5000, chains=4, n_jobs=1)

print(fit)

LResults = fit.extract(permuted=True)
Lmu = LResults['mu']
meanMu = np.mean(Lmu)
plt.figure()
plt.title("Posterior Distribution of mu     Mean = {0:.7}".format(meanMu))
plt.hist(Lmu, bins = 50)
plt.show()
    

Lsigma = LResults['sigma']
meanSigma = np.mean(Lsigma)
plt.figure()   
plt.title("Posterior Distribution of sigma     Mean = {0:.7}".format(meanSigma))
plt.hist(Lsigma, bins = 50)
plt.show()
