import numpy as np
import pystan
import matplotlib.pyplot as plt
import seaborn as sb

Data = [
 [179, 121], [185, 89], [181, 106], [210, 100], [202, 104],
 [197, 91], [227, 107], [208, 101], [188, 99], [189, 112],
 [240, 111], [203, 103], [202, 85], [190, 95], [221, 96],
 [150, 83], [205, 103], [177, 90], [221, 92], [172, 86],
 [190, 87], [218, 101], [205, 100], [208, 120], [224, 98],
 [199, 116], [203, 96], [213, 91], [200, 109], [190, 100],
 [211, 115], [204, 85], [185, 92], [178, 86], [231, 122],
 [206, 84], [197, 93], [236, 93], [213, 106], [205, 103],
 [212, 102], [177, 86], [193, 96], [183, 93], [204, 87],
 [203, 108], [203, 103], [180, 100], [196, 101], [167, 75],
 [202, 106], [182, 98], [225, 109], [219, 115], [205, 102],
 [200, 99], [219, 94], [189, 102], [220, 86], [186, 102],
 [203, 101], [210, 124], [254, 103], [208, 95], [192, 106],
 [176, 91], [173, 97], [195, 100], [189, 98], [220, 115],
 [202, 112], [257, 108], [207, 104], [199, 96], [173, 98],
 [230, 101], [230, 111], [212, 105], [196, 82], [183, 82],
 [230, 103], [207, 112], [225, 101], [192, 104], [175, 83],
 [223, 105], [192, 106], [202, 87], [204, 87], [184, 97],
 [193, 94], [204, 109], [196, 92], [216, 109], [219, 113],
 [256, 117], [210, 86], [217, 103], [206, 87], [213, 105] ]
 
mu = np.mean(Data, axis = 0)
print('mu =\n', mu)
cov = np.cov(np.array(Data).T)
print('cov =\n', cov)
sgms = [cov[0][0]**0.5, cov[1][1]**0.5]
print('sgm =\n', sgms) 

L1 = mu[0] - 5 * sgms[0]
U1 = mu[0] + 5 * sgms[0]
L2 = mu[1] - 5 * sgms[1]
U2 = mu[1] + 5 * sgms[1]
sgm_L1 = 0.0
sgm_U1 = 10 * sgms[0]
sgm_L2 = 0.0
sgm_U2 = 10 * sgms[1]

def f_init():
	return {'mu1': mu[0], 'mu2': mu[1], 'sgm1': sgms[0], 'sgm2': sgms[1],
			'r': 0.0}

sm = pystan.StanModel(file = 'TwoVarNormal.stan')

stan_data = {'N': len(Data), 'Data': Data, 'l1': L1, 'u1': U1, 'l2': L2, 'u2': U2,
			'sgm_l1': sgm_L1, 'sgm_u1': sgm_U1, 
			'sgm_l2': sgm_L2, 'sgm_u2': sgm_U2}

fit = sm.sampling(data = stan_data, init = f_init, n_jobs = 1)

print(fit)

samples_mu1 = fit['mu1']
samples_mu2 = fit['mu2']
plt.title('Posterior Distributions', fontsize = 20)
sb.kdeplot(samples_mu1, label = 'mu1')
sb.kdeplot(samples_mu2, label = 'mu2')
plt.legend(fontsize = 20)
plt.show()

samples_sgm1 = fit['sgm1']
samples_sgm2 = fit['sgm2']
plt.title('Posterior Distributions', fontsize = 20)
sb.kdeplot(samples_sgm1, label = '$\sigma_1$')
sb.kdeplot(samples_sgm2, label = '$\sigma_2$')
plt.legend(fontsize = 20)
plt.show()

samples_r = fit['r']
plt.title('Posterior Distributions', fontsize = 20)
sb.kdeplot(samples_r, label = 'r')
plt.xlabel('r', fontsize = 20)
plt.legend(fontsize = 20)
plt.show()
